@@ -20,14 +20,11 @@ namespace graph {
 
 GlobalState::GlobalState(Graph& g) : graph(g) {
   flat_size = 0;
-  std::set<uint> supp = graph.compute_support();
-  for (uint node_id : supp) {
-    ordered_support.push_back(graph.nodes[node_id].get());
-  }
+  graph.ensure_evaluation_and_inference_readiness();
 
   // initialize unconstrained value types
   // TODO: rename to initialize_unconstrained_value_types
-  for (auto node : ordered_support) {
+  for (auto node : graph.supp) {
     if (node->is_stochastic() and node->node_type == NodeType::OPERATOR) {
       auto sto_node = static_cast<oper::StochasticOperator*>(node);
       sto_node->get_unconstrained_value(true);
@@ -35,7 +32,7 @@ GlobalState::GlobalState(Graph& g) : graph(g) {
   }
 
   // save stochastic and deterministic nodes
-  for (auto node : ordered_support) {
+  for (auto node : graph.supp) {
     if (node->is_stochastic() and !node->is_observed) {
       stochastic_nodes.push_back(node);
       // initialize vals_backup and grads_backup to correct size
@@ -198,13 +195,13 @@ void GlobalState::get_flattened_unconstrained_grads(
   int i = 0;
   for (Node* node : stochastic_nodes) {
     if (node->value.type.variable_type == VariableType::SCALAR) {
-      flattened_grad[i] = node->back_grad1._double;
+      flattened_grad[i] = node->back_grad1;
       i++;
     } else {
       Eigen::VectorXd vector(Eigen::Map<Eigen::VectorXd>(
-          node->back_grad1._matrix.data(), node->back_grad1._matrix.size()));
+          node->back_grad1.data(), node->back_grad1.size()));
       flattened_grad.segment(i, vector.size()) = vector;
-      i += static_cast<int>(node->back_grad1._matrix.size());
+      i += static_cast<int>(node->back_grad1.size());
     }
   }
 }
@@ -214,11 +211,11 @@ double GlobalState::get_log_prob() {
 }
 
 void GlobalState::update_log_prob() {
-  log_prob = graph._full_log_prob(ordered_support);
+  log_prob = graph.full_log_prob();
 }
 
 void GlobalState::update_backgrad() {
-  graph.update_backgrad(ordered_support);
+  graph.eval_and_update_backgrad(graph.supp);
 }
 
 } // namespace graph
