@@ -9,6 +9,7 @@
 #include <random>
 #include <sstream>
 #include <thread>
+#include <variant>
 
 #include "beanmachine/graph/distribution/distribution.h"
 #include "beanmachine/graph/factor/factor.h"
@@ -16,6 +17,7 @@
 #include "beanmachine/graph/operator/operator.h"
 #include "beanmachine/graph/operator/stochasticop.h"
 #include "beanmachine/graph/transform/transform.h"
+#include "beanmachine/graph/util.h"
 
 namespace beanmachine {
 namespace graph {
@@ -64,27 +66,31 @@ std::string ValueType::to_string() const {
 NodeValue::NodeValue(AtomicType type, double value)
     : type(type), _double(value) {
   // don't allow constrained values to get too close to the boundary
-  if (type == AtomicType::POS_REAL) {
-    if (_double < PRECISION) {
-      _double = PRECISION;
-    }
-  } else if (type == AtomicType::NEG_REAL) {
-    if (_double > -PRECISION) {
-      _double = -PRECISION;
-    }
-  } else if (type == AtomicType::PROBABILITY) {
-    if (_double < PRECISION) {
-      _double = PRECISION;
-    } else if (_double > (1 - PRECISION)) {
-      _double = 1 - PRECISION;
-    }
-  } else {
-    // this API is only meant for POS_REAL, NEG_REAL, REAL and PROBABILITY
-    // values
-    if (type != AtomicType::REAL) {
+  switch (type) {
+    case AtomicType::POS_REAL:
+      if (_double < PRECISION) {
+        _double = PRECISION;
+      }
+      break;
+    case AtomicType::NEG_REAL:
+      if (_double > -PRECISION) {
+        _double = -PRECISION;
+      }
+      break;
+    case AtomicType::PROBABILITY:
+      if (_double < PRECISION) {
+        _double = PRECISION;
+      } else if (_double > (1 - PRECISION)) {
+        _double = 1 - PRECISION;
+      }
+      break;
+    case AtomicType::REAL:
+      break;
+    default:
+      // this API is only meant for POS_REAL, NEG_REAL, REAL and PROBABILITY
+      // values
       throw std::invalid_argument(
           "expect probability, pos_real, neg_real or real type with floating point value");
-    }
   }
 }
 
@@ -249,9 +255,9 @@ Node::forward_gradient_scalarops<Eigen::Matrix<double, 1, 2>, Eigen::Matrix2d>(
 void Node::reset_backgrad() {
   assert(value.type.variable_type != graph::VariableType::UNKNOWN);
   if (value.type.variable_type == graph::VariableType::SCALAR) {
-    back_grad1._double = 0;
+    back_grad1 = 0;
   } else {
-    back_grad1._matrix.setZero(value.type.rows, value.type.cols);
+    back_grad1.setZero(value.type.rows, value.type.cols);
   }
 }
 
@@ -297,10 +303,17 @@ std::string Graph::to_string() const {
   return os.str();
 }
 
-void Graph::update_backgrad(std::vector<Node*>& ordered_supp) {
+void Graph::eval_and_update_backgrad(std::vector<Node*>& ordered_supp) {
+  // generator doesn't matter for det nodes
+  // TODO: add default generator
+  std::mt19937 generator(12131);
   for (auto node : ordered_supp) {
     node->reset_backgrad();
+    if (!node->is_stochastic()) {
+      node->eval(generator);
+    }
   }
+
   for (auto it = ordered_supp.rbegin(); it != ordered_supp.rend(); ++it) {
     Node* node = *it;
     if (node->is_stochastic() and node->node_type == NodeType::OPERATOR) {
@@ -358,14 +371,18 @@ void Graph::eval_and_grad(
 }
 
 void Graph::_test_backgrad(
-    std::set<uint>& supp,
+    std::set<uint>& ordered_support_node_ids,
     std::vector<DoubleMatrix*>& grad1) {
-  for (auto it = supp.begin(); it != supp.end(); ++it) {
+  for (auto it = ordered_support_node_ids.begin();
+       it != ordered_support_node_ids.end();
+       ++it) {
     Node* node = nodes[*it].get();
     node->reset_backgrad();
   }
   grad1.clear();
-  for (auto it = supp.rbegin(); it != supp.rend(); ++it) {
+  for (auto it = ordered_support_node_ids.rbegin();
+       it != ordered_support_node_ids.rend();
+       ++it) {
     Node* node = nodes[*it].get();
     if (node->is_stochastic() and node->node_type == NodeType::OPERATOR) {
       auto sto_node = static_cast<oper::StochasticOperator*>(node);
@@ -383,20 +400,22 @@ void Graph::_test_backgrad(
 }
 
 void Graph::test_grad(std::vector<DoubleMatrix*>& grad1) {
-  std::set<uint> supp = compute_support();
-  _test_backgrad(supp, grad1);
+  std::set<uint> ordered_support_node_ids = compute_ordered_support_node_ids();
+  _test_backgrad(ordered_support_node_ids, grad1);
 }
 
 void Graph::eval_and_grad(std::vector<DoubleMatrix*>& grad1, uint seed) {
   std::mt19937 generator(seed);
-  std::set<uint> supp = compute_support();
-  for (auto it = supp.begin(); it != supp.end(); ++it) {
+  std::set<uint> ordered_support_node_ids = compute_ordered_support_node_ids();
+  for (auto it = ordered_support_node_ids.begin();
+       it != ordered_support_node_ids.end();
+       ++it) {
     Node* node = nodes[*it].get();
     if (!node->is_observed) {
       node->eval(generator);
     }
   }
-  _test_backgrad(supp, grad1);
+  _test_backgrad(ordered_support_node_ids, grad1);
 }
 
 void set_value(Eigen::MatrixXd& variable, double value) {
@@ -406,8 +425,7 @@ void set_value(double& variable, double value) {
   variable = value;
 }
 
-template <class T>
-void Graph::gradient_log_prob(uint src_idx, T& grad1, T& grad2) {
+void Graph::gradient_log_prob(uint src_idx, double& grad1, double& grad2) {
   // TODO: As of May 2021, this method is being used for testing only.
   // Refactor code so that we test the code actually being used for the
   // normal functionality of the class.
@@ -419,26 +437,14 @@ void Graph::gradient_log_prob(uint src_idx, T& grad1, T& grad2) {
     throw std::runtime_error(
         "gradient_log_prob only supported on stochastic nodes");
   }
-  uint size = src_node->value.type.cols * src_node->value.type.rows;
-  bool is_src_scalar = (size == 0);
-  // start gradient
-  if (!is_src_scalar) {
-    src_node->Grad1 = Eigen::MatrixXd::Ones(size, 1);
-    src_node->Grad2 = Eigen::MatrixXd::Zero(size, 1);
-  }
   src_node->grad1 = 1;
   src_node->grad2 = 0;
 
-  auto supp = compute_support();
+  auto ordered_support_node_ids = compute_ordered_support_node_ids();
   std::vector<uint> det_nodes;
   std::vector<uint> sto_nodes;
-  std::tie(det_nodes, sto_nodes) = compute_affected_nodes(src_idx, supp);
-  if (!is_src_scalar and det_nodes.size() > 0) {
-    throw std::runtime_error(
-        "compute_gradients has not been implemented for vector source node");
-    // TODO: remove this error message and leave it to compute_gradients
-    // to issue an error if needed.
-  }
+  std::tie(det_nodes, sto_nodes) =
+      compute_affected_nodes(src_idx, ordered_support_node_ids);
   for (auto node_id : det_nodes) {
     Node* node = nodes[node_id].get();
     // passing generator for signature,
@@ -458,34 +464,24 @@ void Graph::gradient_log_prob(uint src_idx, T& grad1, T& grad2) {
   // if we seem to be computing them from scratch when needed.
 
   // end gradient computation reset grads
-  if (!is_src_scalar) {
-    src_node->Grad1.setZero();
-  }
   src_node->grad1 = 0;
   for (auto node_id : det_nodes) {
     Node* node = nodes[node_id].get();
-    if (!is_src_scalar) {
-      node->Grad1.setZero(1, 1);
-      node->Grad2.setZero(1, 1);
-    } else {
-      node->grad1 = node->grad2 = 0;
-    }
+    node->grad1 = node->grad2 = 0;
   }
 }
 
-template void
-Graph::gradient_log_prob<double>(uint src_idx, double& grad1, double& grad2);
-
 double Graph::log_prob(uint src_idx) {
   // TODO: also used in tests only
   Node* src_node = check_node(src_idx, NodeType::OPERATOR);
   if (not src_node->is_stochastic()) {
     throw std::runtime_error("log_prob only supported on stochastic nodes");
   }
-  auto supp = compute_support();
+  auto ordered_support_node_ids = compute_ordered_support_node_ids();
   std::vector<uint> det_nodes;
   std::vector<uint> sto_nodes;
-  std::tie(det_nodes, sto_nodes) = compute_affected_nodes(src_idx, supp);
+  std::tie(det_nodes, sto_nodes) =
+      compute_affected_nodes(src_idx, ordered_support_node_ids);
   for (auto node_id : det_nodes) {
     Node* node = nodes[node_id].get();
     std::mt19937 generator(12131); // seed is irrelevant for deterministic ops
@@ -499,12 +495,11 @@ double Graph::log_prob(uint src_idx) {
   return log_prob;
 }
 
-// TODO: this is the one actually used in code (as opposed to full_log_prob used
-// in testing only, so why the _?)
-double Graph::_full_log_prob(std::vector<Node*>& ordered_supp) {
+double Graph::full_log_prob() {
+  ensure_evaluation_and_inference_readiness();
   double sum_log_prob = 0.0;
   std::mt19937 generator(12131); // seed is irrelevant for deterministic ops
-  for (auto node : ordered_supp) {
+  for (auto node : supp) {
     if (node->is_stochastic()) {
       sum_log_prob += node->log_prob();
       if (node->node_type == NodeType::OPERATOR) {
@@ -518,7 +513,7 @@ double Graph::_full_log_prob(std::vector<Node*>& ordered_supp) {
           // log(f_Y(y)) = log(f_X(x)) + log(|d/dy f^{-1}(y)|)
           //   = node->log_prob() + log_abs_jacobian_determinant()
           // TODO: rename log_abs_jacobian_determinant
-          // to log_abs_jacobian_detesrminant_of_inverse_transform
+          // to log_abs_jacobian_determinant_of_inverse_transform
           //
           // References on Change of Variables in statistics:
           // https://online.stat.psu.edu/stat414/lesson/22/22.2
@@ -534,18 +529,6 @@ double Graph::_full_log_prob(std::vector<Node*>& ordered_supp) {
   return sum_log_prob;
 }
 
-/* TODO: used in testing only; it looks like there has not been a need for it in
- * actual code so far; notheless we can leave it here as it is a natural
- * operation */
-double Graph::full_log_prob() {
-  std::set<uint> supp = compute_support();
-  std::vector<Node*> ordered_supp;
-  for (uint node_id : supp) {
-    ordered_supp.push_back(nodes[node_id].get());
-  }
-  return _full_log_prob(ordered_supp);
-}
-
 // TODO: from now on, we have methods for adding nodes, checking validity,
 // inference and a copy constructor Those are essentially as they should be.
 // Note that methods for determining support and affected nodes are in
@@ -1197,6 +1180,16 @@ std::vector<uint> Graph::get_parent_ids(
   return parent_ids;
 }
 
+void Graph::reindex_nodes() {
+  uint index = 0;
+  for (auto const& node : nodes) {
+    if (node) {
+      node->index = index;
+      index++;
+    }
+  }
+}
+
 Graph::Graph(const Graph& other) {
   // This copy constructor does not copy the inference results (if available)
   // from the source graph.
@@ -1239,5 +1232,198 @@ Graph::Graph(const Graph& other) {
   agg_samples = other.agg_samples;
 }
 
+void Graph::ensure_evaluation_and_inference_readiness() {
+  if (not ready_for_evaluation_and_inference) {
+    pd_begin(ProfilerEvent::NMC_INFER_INITIALIZE);
+    collect_node_ptrs();
+    compute_support();
+    compute_affected_nodes();
+    old_values = std::vector<NodeValue>(nodes.size());
+    pd_finish(ProfilerEvent::NMC_INFER_INITIALIZE);
+    ready_for_evaluation_and_inference = true;
+  }
+}
+
+void Graph::collect_node_ptrs() {
+  for (uint node_id = 0; node_id < static_cast<uint>(nodes.size()); node_id++) {
+    node_ptrs.push_back(nodes[node_id].get());
+  }
+}
+
+void Graph::compute_support() {
+  if (supp.size() != 0) {
+    return;
+  }
+
+  supp.reserve(nodes.size());
+  supp_ids = compute_ordered_support_node_ids();
+  for (uint node_id : supp_ids) {
+    supp.push_back(node_ptrs[node_id]);
+  }
+
+  unobserved_sto_support_index_by_node_id = std::vector<uint>(nodes.size(), 0);
+
+  for (auto node : supp) {
+    bool node_is_not_observed = observed.find(node->index) == observed.end();
+    if (node_is_not_observed) {
+      unobserved_supp.push_back(node);
+      if (node->is_stochastic()) {
+        uint index_of_next_unobserved_sto_supp_node =
+            static_cast<uint>(unobserved_sto_supp.size());
+        unobserved_sto_supp.push_back(node);
+        uint node_id = node->index;
+        assert(unobserved_sto_support_index_by_node_id.size() != 0);
+        unobserved_sto_support_index_by_node_id[node_id] =
+            index_of_next_unobserved_sto_supp_node;
+      }
+    }
+  }
+}
+
+// For every unobserved stochastic node in the graph, we will need to
+// repeatedly know the set of immediate stochastic descendants
+// and intervening deterministic nodes.
+// Because this can be expensive, we compute those sets once and cache them.
+void Graph::compute_affected_nodes() {
+  for (Node* node : unobserved_sto_supp) {
+    auto det_node_ids = util::make_reserved_vector<uint>(nodes.size());
+    auto sto_node_ids = util::make_reserved_vector<uint>(nodes.size());
+    auto det_nodes = util::make_reserved_vector<Node*>(nodes.size());
+    auto sto_nodes = util::make_reserved_vector<Node*>(nodes.size());
+
+    std::tie(det_node_ids, sto_node_ids) =
+        compute_affected_nodes(node->index, supp_ids);
+    for (uint id : det_node_ids) {
+      det_nodes.push_back(node_ptrs[id]);
+    }
+    for (uint id : sto_node_ids) {
+      sto_nodes.push_back(node_ptrs[id]);
+    }
+    det_affected_nodes.push_back(det_nodes);
+    sto_affected_nodes.push_back(sto_nodes);
+    if (_collect_performance_data) {
+      profiler_data.det_supp_count[static_cast<uint>(node->index)] =
+          static_cast<int>(det_nodes.size());
+    }
+  }
+}
+
+const std::vector<Node*>& Graph::get_det_affected_nodes(Node* node) {
+  return det_affected_nodes
+      [unobserved_sto_support_index_by_node_id[node->index]];
+}
+
+const std::vector<Node*>& Graph::get_sto_affected_nodes(Node* node) {
+  return sto_affected_nodes
+      [unobserved_sto_support_index_by_node_id[node->index]];
+}
+
+void Graph::revertibly_set_and_propagate(Node* node, const NodeValue& value) {
+  save_old_value(node);
+  save_old_values(get_det_affected_nodes(node));
+  old_sto_affected_nodes_log_prob =
+      compute_log_prob_of(get_sto_affected_nodes(node));
+  node->value = value;
+  eval(get_det_affected_nodes(node));
+}
+
+void Graph::revert_set_and_propagate(Node* node) {
+  restore_old_value(node);
+  restore_old_values(get_det_affected_nodes(node));
+}
+
+void Graph::save_old_value(const Node* node) {
+  old_values[node->index] = node->value;
+}
+
+void Graph::save_old_values(const std::vector<Node*>& nodes) {
+  pd_begin(ProfilerEvent::NMC_SAVE_OLD);
+  for (Node* node : nodes) {
+    old_values[node->index] = node->value;
+  }
+  pd_finish(ProfilerEvent::NMC_SAVE_OLD);
+}
+
+NodeValue& Graph::get_old_value(const Node* node) {
+  return old_values[node->index];
+}
+
+void Graph::restore_old_value(Node* node) {
+  node->value = old_values[node->index];
+}
+
+void Graph::restore_old_values(const std::vector<Node*>& det_nodes) {
+  pd_begin(ProfilerEvent::NMC_RESTORE_OLD);
+  for (Node* node : det_nodes) {
+    node->value = old_values[node->index];
+  }
+  pd_finish(ProfilerEvent::NMC_RESTORE_OLD);
+}
+
+void Graph::compute_gradients(const std::vector<Node*>& det_nodes) {
+  pd_begin(ProfilerEvent::NMC_COMPUTE_GRADS);
+  for (Node* node : det_nodes) {
+    node->compute_gradients();
+  }
+  pd_finish(ProfilerEvent::NMC_COMPUTE_GRADS);
+}
+
+void Graph::eval(const std::vector<Node*>& det_nodes) {
+  pd_begin(ProfilerEvent::NMC_EVAL);
+  std::mt19937 gen(12131); // seed doesn't matter
+  // because operators are deterministic - TODO: clean it
+  for (Node* node : det_nodes) {
+    node->eval(gen);
+  }
+  pd_finish(ProfilerEvent::NMC_EVAL);
+}
+
+void Graph::clear_gradients(Node* node) {
+  // TODO: eventually we want to have different classes of Node
+  // and have this be a virtual method
+  switch (node->value.type.variable_type) {
+    case VariableType::SCALAR:
+      node->grad1 = 0;
+      node->grad2 = 0;
+      break;
+    case VariableType::BROADCAST_MATRIX:
+    case VariableType::COL_SIMPLEX_MATRIX: {
+      auto rows = node->value._matrix.rows();
+      auto cols = node->value._matrix.cols();
+      node->Grad1 = Eigen::MatrixXd::Zero(rows, cols);
+      node->Grad2 = Eigen::MatrixXd::Zero(rows, cols);
+      break;
+    }
+    default:
+      throw std::runtime_error(
+          "clear_gradients invoked for nodes of an unsupported variable type " +
+          std::to_string(int(node->value.type.variable_type)));
+  }
+}
+
+void Graph::clear_gradients(const std::vector<Node*>& nodes) {
+  pd_begin(ProfilerEvent::NMC_CLEAR_GRADS);
+  for (Node* node : nodes) {
+    clear_gradients(node);
+  }
+  pd_finish(ProfilerEvent::NMC_CLEAR_GRADS);
+}
+
+void Graph::clear_gradients_of_node_and_its_affected_nodes(Node* node) {
+  clear_gradients(node);
+  clear_gradients(get_det_affected_nodes(node));
+  clear_gradients(get_sto_affected_nodes(node));
+}
+
+// Computes the log probability with respect to a given
+// set of stochastic nodes.
+double Graph::compute_log_prob_of(const std::vector<Node*>& sto_nodes) {
+  double log_prob = 0;
+  for (Node* node : sto_nodes) {
+    log_prob += node->log_prob();
+  }
+  return log_prob;
+}
+
 } // namespace graph
 } // namespace beanmachine
