@@ -15,7 +15,9 @@
 #include <set>
 #include <string>
 #include <tuple>
+#include <variant>
 #include <vector>
+#include "beanmachine/graph/double_matrix.h"
 #include "beanmachine/graph/profiler.h"
 
 #define NATURAL_TYPE unsigned long long int
@@ -139,8 +141,7 @@ class NodeValue {
   // contains a non-static data member with a non-trivial special member
   // function (copy/move constructor, copy/move assignment, or destructor), that
   // function is deleted by default in the union and needs to be defined
-  // explicitly by the programmer." Because anonymous unions cannot declare
-  // destructors, we would have to move this move it
+  // explicitly by the programmer."
   Eigen::MatrixXd _matrix;
   Eigen::MatrixXb _bmatrix;
   Eigen::MatrixXn _nmatrix;
@@ -220,50 +221,51 @@ class NodeValue {
   NodeValue(AtomicType type, double value);
 
   NodeValue(const NodeValue& other) : type(other.type) {
-    if (type.variable_type == VariableType::SCALAR) {
-      switch (type.atomic_type) {
-        case AtomicType::UNKNOWN: {
-          throw std::invalid_argument(
-              "Trying to copy an NodeValue of unknown type.");
+    switch (type.variable_type) {
+      case VariableType::SCALAR:
+        switch (type.atomic_type) {
+          case AtomicType::UNKNOWN:
+            throw std::invalid_argument(
+                "Trying to copy an NodeValue of unknown type.");
+          case AtomicType::BOOLEAN:
+            _bool = other._bool;
+            break;
+          case AtomicType::NATURAL:
+            _natural = other._natural;
+            break;
+          default:
+            _double = other._double;
+            break;
         }
-        case AtomicType::BOOLEAN: {
-          _bool = other._bool;
-          break;
+        break;
+      case VariableType::BROADCAST_MATRIX:
+        switch (type.atomic_type) {
+          case AtomicType::BOOLEAN:
+            _bmatrix = other._bmatrix;
+            break;
+          case AtomicType::REAL:
+          case AtomicType::POS_REAL:
+          case AtomicType::NEG_REAL:
+          case AtomicType::PROBABILITY:
+            _matrix = other._matrix;
+            break;
+          case AtomicType::NATURAL:
+            _nmatrix = other._nmatrix;
+            break;
+          default:
+            throw std::invalid_argument(
+                "Trying to copy a MATRIX NodeValue of unsupported type.");
         }
-        case AtomicType::NATURAL: {
-          _natural = other._natural;
-          break;
-        }
-        default: {
-          _double = other._double;
-          break;
-        }
-      }
-    } else if (type.variable_type == VariableType::BROADCAST_MATRIX) {
-      switch (type.atomic_type) {
-        case AtomicType::BOOLEAN:
-          _bmatrix = other._bmatrix;
-          break;
-        case AtomicType::REAL:
-        case AtomicType::POS_REAL:
-        case AtomicType::NEG_REAL:
-        case AtomicType::PROBABILITY:
-          _matrix = other._matrix;
-          break;
-        case AtomicType::NATURAL:
-          _nmatrix = other._nmatrix;
-          break;
-        default:
-          throw std::invalid_argument(
-              "Trying to copy a MATRIX NodeValue of unsupported type.");
-      }
-    } else if (type.variable_type == VariableType::COL_SIMPLEX_MATRIX) {
-      _matrix = other._matrix;
-    } else {
-      throw std::invalid_argument(
-          "Trying to copy a value of unknown VariableType");
+        break;
+      case VariableType::COL_SIMPLEX_MATRIX:
+        _matrix = other._matrix;
+        break;
+      default:
+        throw std::invalid_argument(
+            "Trying to copy a value of unknown VariableType");
     }
   }
+
   NodeValue& operator=(const NodeValue& other) = default;
 
   std::string to_string() const;
@@ -306,6 +308,7 @@ enum class OperatorType {
   TO_POS_REAL,
   COMPLEMENT,
   NEGATE,
+  ELEMENTWISE_MULTIPLY,
   EXP,
   EXPM1,
   MULTIPLY,
@@ -319,8 +322,10 @@ enum class OperatorType {
   LOG,
   POW,
   LOG1MEXP,
+  TRANSPOSE,
   MATRIX_MULTIPLY,
   MATRIX_SCALE,
+  MATRIX_ADD,
   TO_PROBABILITY,
   INDEX,
   COLUMN_INDEX,
@@ -331,6 +336,9 @@ enum class OperatorType {
   TO_NEG_REAL,
   CHOICE,
   TO_INT,
+  CHOLESKY,
+  MATRIX_EXP,
+  LOG_PROB,
 };
 
 enum class DistributionType {
@@ -343,6 +351,7 @@ enum class DistributionType {
   DIRICHLET,
   FLAT,
   NORMAL,
+  LOG_NORMAL,
   HALF_NORMAL,
   HALF_CAUCHY,
   STUDENT_T,
@@ -350,6 +359,10 @@ enum class DistributionType {
   GAMMA,
   BIMIXTURE,
   CATEGORICAL,
+  POISSON,
+  GEOMETRIC,
+  CAUCHY,
+  DUMMY,
 };
 
 enum class FactorType {
@@ -363,18 +376,13 @@ enum class NodeType {
   DISTRIBUTION = 2,
   OPERATOR = 3,
   FACTOR = 4,
-  MAX
+  MAX = 5
 };
 
 enum class InferenceType { UNKNOWN = 0, REJECTION = 1, GIBBS, NMC };
 
 enum class AggregationType { UNKNOWN = 0, NONE = 1, MEAN };
 
-struct DoubleMatrix {
-  double _double;
-  Eigen::MatrixXd _matrix;
-};
-
 struct InferConfig {
   bool keep_log_prob;
   double path_length;
@@ -495,8 +503,27 @@ class Node {
   // evaluate the node and store the result in `value` if appropriate
   // eval may involve sampling and that's why we need the random number engine
   virtual void eval(std::mt19937& gen) = 0;
-  // populate the derivatives
+
+  // Computes the first and second gradients of this node
+  // with respect to some (unspecified -- see below) variable.
+  // More specifically, it uses the values stored in this node's input
+  // nodes grad1 and grad2 fields (or Grad1 and Grad2 if they are matrices)
+  // to compute the value of this node's own gradient fields
+  // (again grad1, grad2 or Grad1 and Grad2).
+  // Note that this method does *not* compute the gradient of
+  // this node with respect to its inputs,
+  // but with respect to some (possibly distant) variable.
+  // The method is neutral regarding which variable this is,
+  // and simply computes its own gradient with respect
+  // to the same variable its input node gradients are
+  // with respect to.
+  // In the planned refactoring of BMG's autograd (as of May 2022)
+  // this should be replaced by the more fundamental and modular function
+  // computing a node's gradient with respect to its own inputs,
+  // which should then be used as needed in
+  // applications of the chain rule.
   virtual void compute_gradients() {}
+
   /*
   Gradient backward propagation: computes the 1st-order gradient update and
   add it to the parent's back_grad1.
@@ -678,7 +705,22 @@ struct Graph {
   std::vector<double>& get_elbo() {
     return elbo_vals;
   }
-  std::set<uint> compute_support();
+  /*
+  The support of a graph is the set of operator and factor nodes that
+  are needed to determine the value of query and observed variables. In other
+  words, it is the set of queried and observed variables themselves plus their
+  ancestors that are operator and factor nodes.
+  */
+  std::set<uint> compute_ordered_support_node_ids();
+  /*
+  The full support of a graph includes *all* nodes, including
+  distribution nodes and constant nodes, that are needed to determine the value
+  of query and observed variables
+  */
+  std::set<uint> compute_full_ordered_support_node_ids();
+
+  std::set<uint> compute_ordered_support_node_ids_with_operators_only_choice(
+      bool operator_factor_only);
 
   /*
   Computes the _affected nodes_ of a root node.
@@ -711,24 +753,60 @@ struct Graph {
   typically re-computing the *values* of deterministic nodes,
   and re-computing the *probability* of stochastic nodes.
 
+  The method guarantees to return the deterministic node in
+  a topologically sorted order from the source.
+  This ensures that evaluating these nodes individually
+  in the given order produces the correct global result.
+
   :param node_id: the id (index in topological order) of the node for which
   we are computing the descendants
-  :param support: the set of indices of the distribution support.
-  :returns: vector of intervening deterministic nodes and vector of stochastic
-  nodes that are operators and immediate stochastic descendants of the current
-  node and in the support
-  (that is to say, we don't return descendants of stochastic
-  descendants). The current node is included in result if it is in support and
-  is stochastic.
+  :param ordered_support_node_ids: the (ordered) set of indices of the
+  distribution support.
+  :returns: vector of intervening operator deterministic
+  nodes and vector of stochastic nodes that are operators and immediate
+  stochastic descendants of the current node and in the support (that is to say,
+  we don't return descendants of stochastic descendants). The current node is
+  included in result if it is in support and is stochastic.
   */
   std::tuple<std::vector<uint>, std::vector<uint>> compute_affected_nodes(
       uint node_id,
-      const std::set<uint>& support);
+      const std::set<uint>& ordered_support_node_ids);
+
+  /*
+  This function is almost the same as `compute_affected_nodes` above, with
+  a few key differences:
+  1. the deterministic nodes among the nodes returned by
+`compute_affected_nodes` only include operator deterministic nodes, which have
+  values which need to be re-calculated during inference. This function returns
+  *all* the deterministic nodes between the current node and its stochastic
+  children, including distribution nodes, constants, etc
+  2. `compute_affected_nodes` includes the current stochastic node, while this
+  function only includes its children
+  :param node_id: the id (index in topological order) of the node for which we
+  are computing the descendants
+  :param ordered_support_node_ids: the set of indices of the distribution
+  support.
+  :returns: vector of all intermediate deterministic nodes and vector of
+  stochastic nodes and immediate stochastic descendants of the current node and
+  in the support (that is to say, we don't return descendants of stochastic
+  descendants). The current node is included in result if it is in support and
+  is stochastic.
+  */
+  std::tuple<std::vector<uint>, std::vector<uint>> compute_children(
+      uint node_id,
+      const std::set<uint>& ordered_support_node_ids);
+
+  std::tuple<std::vector<uint>, std::vector<uint>>
+  _compute_nodes_until_stochastic(
+      uint node_id,
+      const std::set<uint>& ordered_support_node_ids,
+      bool affected_only,
+      bool include_root_node);
 
   std::tuple<std::vector<uint>, std::vector<uint>> compute_ancestors(
       uint node_id);
 
-  void update_backgrad(std::vector<Node*>& ordered_supp);
+  void eval_and_update_backgrad(std::vector<Node*>& ordered_supp);
   /*
   Evaluate the target node and compute its gradient w.r.t. source_node
   (used for unit tests)
@@ -748,9 +826,9 @@ struct Graph {
       double& grad1,
       double& grad2);
   /*
-  Evaluate all nodes in the support and compute their gradients in backward
-  mode. (used for unit tests)
-  :param grad1: Output value of first gradient.
+  Evaluate all nodes in the support and compute their gradients in
+  backward mode. (used for unit tests) :param grad1: Output value of first
+  gradient.
   :param seed: Random number generator seed.
   */
   void eval_and_grad(std::vector<DoubleMatrix*>& grad1, uint seed = 5123412);
@@ -764,22 +842,19 @@ struct Graph {
 
   /*
   Evaluate the deterministic descendants of the source node and compute
-  the logprob_gradient of all stochastic descendants in the support including
-  the source node.
+  the logprob_gradient of all stochastic descendants in the support
+  including the source node.
 
   :param src_idx: The index of the node to evaluate the gradients w.r.t., must
                   be a vector valued node.
-  :param grad1: Output value of first gradient (double), or gradient vector
-                (Eigen::MatrixXd)
-  :param grad2: Output value of the second gradient (double), or the diagonal
-                terms of the gradient matrix (Eigen::MatrixXd).
+  :param grad1: Output value of first gradient (double)
+  :param grad2: Output value of the second gradient (double)
   */
-  template <class T>
-  void gradient_log_prob(uint src_idx, T& grad1, T& grad2);
+  void gradient_log_prob(uint src_idx, double& grad1, double& grad2);
   /*
   Evaluate the deterministic descendants of the source node and compute
-  the sum of logprob of all stochastic descendants in the support including
-  the source node.
+  the sum of logprob of all stochastic descendants in the support
+  including the source node.
 
   :param src_idx: source node
   :returns: The sum of log_prob of source node and all stochastic descendants.
@@ -864,19 +939,22 @@ struct Graph {
       uint elbo_samples);
   /*
   Evaluate the full log probability over the support of the graph.
-  :param ordered_supp: node pointers in the support in topological order.
-  :returns: The sum of log_prob of stochastic nodes in the support.
+  :param ordered_supp: node pointers in the support in topological
+  order.
+  :returns: The sum of log_prob of stochastic nodes in the
+  support.
   */
 
   // TODO: Review what members of this class can be made static.
 
-  static double _full_log_prob(std::vector<Node*>& ordered_supp);
   void collect_log_prob(double log_prob);
   std::vector<double> log_prob_vals;
   std::vector<std::vector<double>> log_prob_allchains;
   std::map<TransformType, std::unique_ptr<Transformation>>
       common_transformations;
-  void _test_backgrad(std::set<uint>& supp, std::vector<DoubleMatrix*>& grad1);
+  void _test_backgrad(
+      std::set<uint>& ordered_support_node_ids,
+      std::vector<DoubleMatrix*>& grad1);
 
   ProfilerData profiler_data;
   bool _collect_performance_data = false;
@@ -895,6 +973,127 @@ struct Graph {
       profiler_data.finish(kind);
     }
   }
+
+  void reindex_nodes();
+
+  // members brought in from MH class since they are really Graph properties
+ public:
+  // A graph maintains of a vector of nodes; the index into that vector is
+  // the id of the node. We often need to translate from node ids into node
+  // pointers; to do so quickly we obtain the address of
+  // every node in the graph up front and then look it up when we need it.
+  std::vector<Node*> node_ptrs;
+
+  // Every node in the graph has a value; when we propose a new graph state,
+  // we update the values. If we then reject the proposed new state, we need
+  // to restore the values. This vector stores the original values of the
+  // nodes that we change during the proposal step.
+  // We do the same for the log probability of the stochastic nodes
+  // affected by the last revertible set and propagate operation
+  // see (revertibly_set_and_propagate method).
+  std::vector<NodeValue> old_values;
+  double old_sto_affected_nodes_log_prob;
+
+  // The support is the set of all nodes in the graph that are queried or
+  // observed, directly or indirectly. We keep both node ids and node pointer
+  // forms.
+  std::set<uint> supp_ids;
+  std::vector<Node*> supp;
+
+  // Nodes in support that are not directly observed. Note that
+  // the order of nodes in this vector matters! We must enumerate
+  // them in order from lowest node identifier to highest.
+  std::vector<Node*> unobserved_supp;
+
+  // Nodes in unobserved_supp that are stochastic; similarly, order matters.
+  std::vector<Node*> unobserved_sto_supp;
+
+  // A vector containing the index of a node in vector unobserved_sto_supp for
+  // each node_id. Since not all nodes are in unobserved_sto_support, some
+  // elements of this vector should never be accessed.
+  std::vector<uint> unobserved_sto_support_index_by_node_id;
+
+  // These vectors are the same size as unobserved_sto_support.
+  // The i-th elements are vectors of nodes which are
+  // respectively the vector of
+  // the immediate stochastic descendants of node with index i in the
+  // support, and the vector of the intervening deterministic nodes
+  // between the i-th node and its immediate stochastic descendants.
+  // In other words, these are the cached results of
+  // invoking graph::compute_affected_nodes
+  // for each node.
+  std::vector<std::vector<Node*>> sto_affected_nodes;
+  std::vector<std::vector<Node*>> det_affected_nodes;
+
+  bool ready_for_evaluation_and_inference = false;
+
+  // Methods
+
+  // Ensures graph is ready for evaluation and inference (by building
+  // intermediate internal data structures).
+  // The data structures are built only the first time the method is invoked.
+  // After that, the method is simply ensuring they are built.
+  // Note that this assumes the graph has not changed since the last invocation.
+  // If the graph does change, client code can set field "ready" to false
+  // and then invoke this method.
+  void ensure_evaluation_and_inference_readiness();
+
+  void collect_node_ptrs();
+
+  void compute_support();
+
+  void ensure_all_nodes_are_supported();
+
+  void compute_initial_values();
+
+  void compute_affected_nodes();
+
+  void generate_sample();
+
+  void collect_samples(uint num_samples, InferConfig infer_config);
+
+  void collect_sample(InferConfig infer_config);
+
+  const std::vector<Node*>& get_det_affected_nodes(Node* node);
+
+  const std::vector<Node*>& get_sto_affected_nodes(Node* node);
+
+  // Sets a given node to a new value and
+  // updates its deterministically affected nodes.
+  // Does so in a revertible manner by saving old values and old stochastic
+  // affected nodes log prob.
+  // Old values can be accessed through get_old_* methods.
+  // The reversion is executed by invoking revert_set_and_propagate.
+  void revertibly_set_and_propagate(Node* node, const NodeValue& value);
+
+  // Revert the last revertibly_set_and_propagate
+  void revert_set_and_propagate(Node* node);
+
+  void save_old_value(const Node* node);
+
+  void save_old_values(const std::vector<Node*>& nodes);
+
+  NodeValue& get_old_value(const Node* node);
+
+  double get_old_sto_affected_nodes_log_prob() {
+    return old_sto_affected_nodes_log_prob;
+  }
+
+  void restore_old_value(Node* node);
+
+  void restore_old_values(const std::vector<Node*>& det_nodes);
+
+  void compute_gradients(const std::vector<Node*>& det_nodes);
+
+  void eval(const std::vector<Node*>& det_nodes);
+
+  void clear_gradients(Node* node);
+
+  void clear_gradients(const std::vector<Node*>& nodes);
+
+  void clear_gradients_of_node_and_its_affected_nodes(Node* node);
+
+  double compute_log_prob_of(const std::vector<Node*>& sto_nodes);
 };
 
 } // namespace graph
