@@ -3,7 +3,7 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
-from beanmachine.ppl.model.statistical_model import random_variable, functional
+from beanmachine.ppl.model.statistical_model import functional, random_variable
 
 allowed_functions = {dict, list, set, super, random_variable, functional}
 
