@@ -13,7 +13,7 @@ import numpy as np
 import torch
 import torch.distributions as dist
 from beanmachine.ppl.compiler.bmg_nodes import BMGNode, ConstantNode
-from beanmachine.ppl.compiler.hint import log1mexp, math_log1mexp
+from beanmachine.ppl.compiler.execution_context import ExecutionContext
 from beanmachine.ppl.utils.memoize import memoize
 
 
@@ -29,6 +29,8 @@ supported_float_types = {np.longdouble, np.float16, np.float32, np.float64, floa
 supported_int_types = {np.int16, np.int32, np.int64, np.int8, np.longlong}
 supported_int_types |= {np.uint16, np.uint32, np.uint64, np.uint8, np.ulonglong, int}
 
+_empty_context = ExecutionContext()
+
 
 class BMGraphBuilder:
 
@@ -64,10 +66,13 @@ class BMGraphBuilder:
 
     _pd: Optional[prof.ProfilerData]
 
-    def __init__(self) -> None:
+    execution_context: ExecutionContext
+
+    def __init__(self, execution_context: ExecutionContext = _empty_context) -> None:
         self._nodes = {}
         self._node_counter = 0
         self._pd = None
+        self.execution_context = execution_context
 
     def _begin(self, s: str) -> None:
         pd = self._pd
@@ -90,24 +95,8 @@ class BMGraphBuilder:
     # By memoizing almost all the "add" methods we ensure that
     # the graph is deduplicated automatically.
     #
-    # The "handle" methods, in contrast, conditionally create new
-    # graph nodes only when required because an operation on a
-    # stochastic value must be accumulated into the graph.
-    #
-    # For example, if we have a call to handle_addition where all
-    # operands are ordinary floats (or constant graph nodes)
-    # then there is no need to add a new node to the graph. But
-    # if we have an addition of 1.0 to a stochastic node -- perhaps
-    # a sample node, or perhaps some other graph node that eventually
-    # involves a sample node -- then we need to construct a new
-    # addition node, which is then returned and becomes the value
-    # manipulated by the executing lifted program.
-    #
-    # TODO: The code in the "handle" methods which folds operations
-    # on constant nodes and regular values is a holdover from an
-    # earlier prototyping stage in which all values were lifted to
-    # graph nodes. These scenarios should now be impossible, and we
-    # should take a work item to remove this now-unnecessary code.
+    # TODO: This code does constant folding as well as deduplication,
+    # but that could be moved to a later optimization pass.
 
     def add_node(self, node: BMGNode) -> BMGNode:
         # TODO: This should be private
@@ -686,7 +675,7 @@ class BMGraphBuilder:
     @memoize
     def add_power(self, left: BMGNode, right: BMGNode) -> BMGNode:
         if isinstance(left, ConstantNode) and isinstance(right, ConstantNode):
-            return self.add_constant(left.value ** right.value)
+            return self.add_constant(left.value**right.value)
         node = bn.PowerNode(left, right)
         self.add_node(node)
         return node
@@ -754,6 +743,12 @@ class BMGraphBuilder:
         self.add_node(node)
         return node
 
+    @memoize
+    def add_sum(self, operand: BMGNode) -> BMGNode:
+        node = bn.SumNode(operand)
+        self.add_node(node)
+        return node
+
     @memoize
     def add_to_real(self, operand: BMGNode) -> BMGNode:
         if isinstance(operand, bn.RealNode):
@@ -812,6 +807,12 @@ class BMGraphBuilder:
         self.add_node(node)
         return node
 
+    @memoize
+    def add_cholesky(self, operand: BMGNode) -> BMGNode:
+        node = bn.CholeskyNode(operand)
+        self.add_node(node)
+        return node
+
     @memoize
     def add_exp(self, operand: BMGNode) -> BMGNode:
         if isinstance(operand, bn.ConstantTensorNode):
@@ -822,6 +823,14 @@ class BMGraphBuilder:
         self.add_node(node)
         return node
 
+    @memoize
+    def add_exp2(self, operand: BMGNode) -> BMGNode:
+        if isinstance(operand, ConstantNode):
+            return self.add_constant(torch.exp2(operand.value))
+        node = bn.Exp2Node(operand)
+        self.add_node(node)
+        return node
+
     @memoize
     def add_expm1(self, operand: BMGNode) -> BMGNode:
         if isinstance(operand, bn.ConstantTensorNode):
@@ -860,16 +869,54 @@ class BMGraphBuilder:
         self.add_node(node)
         return node
 
+    @memoize
+    def add_log10(self, operand: BMGNode) -> BMGNode:
+        if isinstance(operand, bn.ConstantNode):
+            return self.add_constant(torch.log10(operand.value))
+        node = bn.Log10Node(operand)
+        self.add_node(node)
+        return node
+
+    @memoize
+    def add_log1p(self, operand: BMGNode) -> BMGNode:
+        if isinstance(operand, bn.ConstantNode):
+            return self.add_constant(torch.log1p(operand.value))
+        node = bn.Log1pNode(operand)
+        self.add_node(node)
+        return node
+
+    @memoize
+    def add_log2(self, operand: BMGNode) -> BMGNode:
+        if isinstance(operand, bn.ConstantNode):
+            return self.add_constant(torch.log2(operand.value))
+        node = bn.Log2Node(operand)
+        self.add_node(node)
+        return node
+
     @memoize
     def add_log1mexp(self, operand: BMGNode) -> BMGNode:
         if isinstance(operand, bn.ConstantTensorNode):
-            return self.add_constant(log1mexp(operand.value))
+            return self.add_constant((1 - operand.value.exp()).log())
         if isinstance(operand, ConstantNode):
-            return self.add_constant(math_log1mexp(operand.value))
+            return self.add_constant(math.log(1 - math.exp(operand.value)))
         node = bn.Log1mexpNode(operand)
         self.add_node(node)
         return node
 
+    @memoize
+    def add_transpose(self, operand: BMGNode) -> bn.TransposeNode:
+        node = bn.TransposeNode(operand)
+        self.add_node(node)
+        return node
+
+    @memoize
+    def add_squareroot(self, operand: BMGNode) -> BMGNode:
+        if isinstance(operand, bn.ConstantNode):
+            return self.add_constant(torch.sqrt(operand.value))
+        node = bn.SquareRootNode(operand)
+        self.add_node(node)
+        return node
+
     @memoize
     def add_tensor(self, size: torch.Size, *data: BMGNode) -> bn.TensorNode:
         node = bn.TensorNode(list(data), size)
@@ -904,6 +951,12 @@ class BMGraphBuilder:
         self.add_node(node)
         return node
 
+    @memoize
+    def add_logaddexp(self, left: bn.BMGNode, right: bn.BMGNode) -> bn.LogAddExpNode:
+        node = bn.LogAddExpNode(left, right)
+        self.add_node(node)
+        return node
+
     @memoize
     def add_switch(self, *elements: BMGNode) -> bn.SwitchNode:
         # TODO: Verify that the list is well-formed.
