@@ -891,6 +891,21 @@ class RShiftNode(BinaryOperatorNode):
         BinaryOperatorNode.__init__(self, left, right)
 
 
+# LogAddExpNode represents a call to the binary operator logAddExp in the original
+# code. It is transformed into LogSumExpNode.
+
+
+class LogAddExpNode(BinaryOperatorNode):
+    """This class represents the LogAddExp operation: for values v_1, v_2
+    we compute log(exp(v_1) + exp(v_2))"""
+
+    def __init__(self, left: BMGNode, right: BMGNode):
+        BinaryOperatorNode.__init__(self, left, right)
+
+    def __str__(self) -> str:
+        return "LogAddExp({self.left}, {self.right})"
+
+
 class SwitchNode(BMGNode):
 
     """This class represents a point in a program where there are
@@ -1046,6 +1061,17 @@ class UnaryOperatorNode(OperatorNode, metaclass=ABCMeta):
         return self.inputs[0]
 
 
+class CholeskyNode(UnaryOperatorNode):
+    """This represents an Cholesky operation; it is generated when
+    a model contains calls to Tensor.cholesky."""
+
+    def __init__(self, operand: BMGNode):
+        UnaryOperatorNode.__init__(self, operand)
+
+    def __str__(self) -> str:
+        return "Cholesky(" + str(self.operand) + ")"
+
+
 class ExpNode(UnaryOperatorNode):
     """This represents an exponentiation operation; it is generated when
     a model contains calls to Tensor.exp or math.exp."""
@@ -1057,6 +1083,14 @@ class ExpNode(UnaryOperatorNode):
         return "Exp(" + str(self.operand) + ")"
 
 
+class Exp2Node(UnaryOperatorNode):
+    def __init__(self, operand: BMGNode):
+        UnaryOperatorNode.__init__(self, operand)
+
+    def __str__(self) -> str:
+        return "Exp2(" + str(self.operand) + ")"
+
+
 class ExpM1Node(UnaryOperatorNode):
     """This represents the operation exp(x) - 1; it is generated when
     a model contains calls to Tensor.expm1."""
@@ -1096,12 +1130,41 @@ class LogNode(UnaryOperatorNode):
         return "Log(" + str(self.operand) + ")"
 
 
-# TODO: replace "log" with "log1mexp" as needed below and update defs
+class Log10Node(UnaryOperatorNode):
+    def __init__(self, operand: BMGNode):
+        UnaryOperatorNode.__init__(self, operand)
+
+    def __str__(self) -> str:
+        return "Log10(" + str(self.operand) + ")"
+
+
+class Log1pNode(UnaryOperatorNode):
+    def __init__(self, operand: BMGNode):
+        UnaryOperatorNode.__init__(self, operand)
+
+    def __str__(self) -> str:
+        return "Log1p(" + str(self.operand) + ")"
+
+
+class Log2Node(UnaryOperatorNode):
+    def __init__(self, operand: BMGNode):
+        UnaryOperatorNode.__init__(self, operand)
+
+    def __str__(self) -> str:
+        return "Log2(" + str(self.operand) + ")"
+
+
+class SquareRootNode(UnaryOperatorNode):
+    def __init__(self, operand: BMGNode):
+        UnaryOperatorNode.__init__(self, operand)
+
+    def __str__(self) -> str:
+        return "SquareRoot(" + str(self.operand) + ")"
 
 
 class Log1mexpNode(UnaryOperatorNode):
-    """This represents a log1mexp operation; it is generated when
-    a model contains calls to log1mexp or math_log1mexp."""
+    """This represents a log1mexp operation; it is generated as an
+    optimization when a graph contains x -> exp -> complement -> log"""
 
     def __init__(self, operand: BMGNode):
         UnaryOperatorNode.__init__(self, operand)
@@ -1110,6 +1173,17 @@ class Log1mexpNode(UnaryOperatorNode):
         return "Log1mexp(" + str(self.operand) + ")"
 
 
+class TransposeNode(UnaryOperatorNode):
+    """This represents a transpose operation; it is generated when
+    a model contains calls to transpose or Tensor.transpose"""
+
+    def __init__(self, operand: BMGNode):
+        UnaryOperatorNode.__init__(self, operand)
+
+    def __str__(self) -> str:
+        return "Transpose(" + str(self.operand) + ")"
+
+
 # BMG supports three different kinds of negation:
 
 # * The "complement" node with a Boolean operand has the semantics
@@ -1245,6 +1319,14 @@ class SampleNode(UnaryOperatorNode):
         return "Sample(" + str(self.operand) + ")"
 
 
+class SumNode(UnaryOperatorNode):
+    def __init__(self, operand: BMGNode):
+        UnaryOperatorNode.__init__(self, operand)
+
+    def __str__(self) -> str:
+        return "Sum(" + str(self.operand) + ")"
+
+
 class ToRealNode(UnaryOperatorNode):
     def __init__(self, operand: BMGNode):
         UnaryOperatorNode.__init__(self, operand)
