@@ -35,13 +35,15 @@ from beanmachine.ppl.compiler.lattice_typer import LatticeTyper
 # what their inputs are.
 
 _known_requirements: Dict[type, List[bt.Requirement]] = {
-    bn.Observation: [bt.AnyRequirement()],
-    bn.Query: [bt.AnyRequirement()],
+    bn.Observation: [bt.any_requirement],
+    bn.Query: [bt.any_requirement],
     # Distributions
     bn.BernoulliLogitNode: [bt.Real],
     bn.BernoulliNode: [bt.Probability],
     bn.BetaNode: [bt.PositiveReal, bt.PositiveReal],
     bn.BinomialNode: [bt.Natural, bt.Probability],
+    # TODO: We don't enforce squareness or positive-definite-ness for Cholesky
+    bn.CholeskyNode: [bt.any_real_matrix],
     bn.GammaNode: [bt.PositiveReal, bt.PositiveReal],
     bn.HalfCauchyNode: [bt.PositiveReal],
     bn.NormalNode: [bt.Real, bt.PositiveReal],
@@ -50,6 +52,7 @@ _known_requirements: Dict[type, List[bt.Requirement]] = {
     # Operators
     bn.LogisticNode: [bt.Real],
     bn.Log1mexpNode: [bt.NegativeReal],
+    bn.MatrixMultiplicationNode: [bt.any_real_matrix, bt.any_real_matrix],
     bn.PhiNode: [bt.Real],
     bn.ToIntNode: [bt.upper_bound(bt.Real)],
     bn.ToNegativeRealNode: [bt.Real],
@@ -59,7 +62,7 @@ _known_requirements: Dict[type, List[bt.Requirement]] = {
 }
 
 
-def _requirements_valid(node: bn.BMGNode, reqs: List[bt.Requirement]):
+def _requirements_valid(node: bn.BMGNode, reqs: List[bt.Requirement]) -> bool:
     return len(reqs) == len(node.inputs) and not any(
         r in bt._invalid_requirement_types for r in reqs
     )
@@ -90,8 +93,10 @@ class EdgeRequirements:
             bn.LogNode: self._requirements_log,
             bn.LogSumExpNode: self._requirements_logsumexp,
             bn.LogSumExpVectorNode: self._requirements_logsumexp_vector,
-            bn.MultiplicationNode: self._requirements_multiplication,
+            # TODO: bn.MatrixMultiplyNode: self._requirements_matrix_multiply,
+            # see comment above
             bn.MatrixScaleNode: self._requirements_matrix_scale,
+            bn.MultiplicationNode: self._requirements_multiplication,
             bn.NegateNode: self._requirements_exp_neg,
             bn.PowerNode: self._requirements_power,
             bn.SampleNode: self._same_as_output,
@@ -429,7 +434,7 @@ class EdgeRequirements:
         if input_count == 0:
             result = []
         elif self.typer[node] == bt.Untypable:
-            result = [bt.AnyRequirement()] * input_count
+            result = [bt.any_requirement] * input_count
         else:
             t = type(node)
             if t in _known_requirements:
@@ -437,7 +442,7 @@ class EdgeRequirements:
             elif t in self._dispatch:
                 result = self._dispatch[t](node)
             else:
-                result = [bt.AnyRequirement()] * input_count
+                result = [bt.any_requirement] * input_count
 
         assert _requirements_valid(node, result)
         return result
