@@ -7,7 +7,7 @@ from abc import abstractmethod
 from typing import Any, Union
 
 import torch
-from beanmachine.ppl.utils.memoize import MemoizedClass, memoize
+from beanmachine.ppl.utils.memoize import memoize, MemoizedClass
 from torch import Size
 
 
@@ -257,7 +257,7 @@ PositiveReal = PositiveRealMatrix(1, 1)
 NegativeReal = NegativeRealMatrix(1, 1)
 Real = RealMatrix(1, 1)
 Tensor = BMGLatticeType("T", "tensor")
-top = Tensor
+top: BMGLatticeType = Tensor
 # This is not a real lattice type; rather, this is a marker to indicate
 # that the node cannot have a lattice type assigned to it in the first
 # place because we lack BMG typing rules for it.
@@ -553,7 +553,7 @@ def _type_of_matrix(v: torch.Tensor) -> BMGLatticeType:
     # treat it as a single value.
 
     if elements == 1:
-        return type_of_value(float(v))  # pyre-fixme
+        return type_of_value(float(v))
 
     # We have more than one element. What's the shape?
 
@@ -737,12 +737,26 @@ class BaseRequirement:
         self.long_name = long_name
 
 
-# TODO: Memoize these, remove memoization of construction functions below.
 class AnyRequirement(BaseRequirement):
     def __init__(self) -> None:
         BaseRequirement.__init__(self, "any", "any")
 
 
+any_requirement = AnyRequirement()
+
+
+class AnyRealMatrix(BaseRequirement):
+    # BMG's matrix multiplication node requires that both inputs be
+    # a real, positive real, negative real or probability matrix.
+    # This singleton represents that requirement.
+    def __init__(self) -> None:
+        BaseRequirement.__init__(self, "ARM", "any real matrix")
+
+
+any_real_matrix = AnyRealMatrix()
+
+
+# TODO: Memoize these, remove memoization of construction functions below.
 class UpperBound(BaseRequirement):
     bound: BMGLatticeType
 
@@ -774,7 +788,7 @@ def upper_bound(bound: Requirement) -> BaseRequirement:
         return upper_bound(bound.bound)
     if isinstance(bound, BMGLatticeType):
         return UpperBound(bound)
-    assert isinstance(bound, AnyRequirement)
+    assert bound is any_requirement
     return bound
 
 
@@ -798,8 +812,10 @@ def requirement_to_type(r: Requirement) -> BMGLatticeType:
 
 def must_be_matrix(r: Requirement) -> bool:
     """Does the requirement indicate that the edge must be a matrix?"""
-    if isinstance(r, AnyRequirement):
+    if r is any_requirement:
         return False
+    if r is any_real_matrix:
+        return True
     if isinstance(r, AlwaysMatrix):
         return True
     t = requirement_to_type(r)
