@@ -17,11 +17,24 @@ from beanmachine.ppl.compiler.bm_graph_builder import BMGraphBuilder
 from beanmachine.ppl.compiler.bmg_node_types import is_supported_by_bmg
 from beanmachine.ppl.compiler.bmg_requirements import EdgeRequirements
 from beanmachine.ppl.compiler.error_report import ErrorReport, Violation
+from beanmachine.ppl.compiler.fix_problem import GraphFixer
 from beanmachine.ppl.compiler.graph_labels import get_edge_labels
 from beanmachine.ppl.compiler.internal_error import InternalError
 from beanmachine.ppl.compiler.lattice_typer import LatticeTyper
 
 
+def _is_real_matrix(t: bt.BMGLatticeType) -> bool:
+    return any(
+        isinstance(t, m)
+        for m in {
+            bt.RealMatrix,
+            bt.PositiveRealMatrix,
+            bt.NegativeRealMatrix,
+            bt.ProbabilityMatrix,
+        }
+    )
+
+
 class RequirementsFixer:
     """This class takes a Bean Machine Graph builder and attempts to
     fix violations of BMG type system requirements.
@@ -46,8 +59,10 @@ class RequirementsFixer:
 
     def _type_meets_requirement(self, t: bt.BMGLatticeType, r: bt.Requirement) -> bool:
         assert t != bt.Untypable
-        if isinstance(r, bt.AnyRequirement):
+        if r is bt.any_requirement:
             return True
+        if r is bt.any_real_matrix:
+            return _is_real_matrix(t)
         if isinstance(r, bt.UpperBound):
             return bt.supremum(t, r.bound) == r.bound
         if isinstance(r, bt.AlwaysMatrix):
@@ -61,6 +76,10 @@ class RequirementsFixer:
             return self._typer.is_matrix(node) and self._type_meets_requirement(
                 lattice_type, r.bound
             )
+        if r is bt.any_real_matrix:
+            return self._typer.is_matrix(node) and self._type_meets_requirement(
+                lattice_type, r
+            )
         return self._type_meets_requirement(lattice_type, r)
 
     def _meet_constant_requirement(
@@ -84,8 +103,19 @@ class RequirementsFixer:
         # a reachable but untypable constant node.  See comment in fix_unsupported
         # regarding UntypedConstantNode support.
 
+        if requirement is bt.any_real_matrix:
+            if _is_real_matrix(it):
+                # It's already an R, R+, R- or P matrix, but it might be a single
+                # value. Ensure that it is marked as a matrix, not a single value.
+                assert isinstance(it, bt.BMGMatrixType)
+                return self.bmg.add_constant_of_matrix_type(node.value, it)
+            else:
+                # It's some other type, such as Boolean or Natural matrix.
+                # Emit the value as the equivalent real matrix:
+                return self.bmg.add_real_matrix(node.value)
+
         if self._type_meets_requirement(it, bt.upper_bound(requirement)):
-            if isinstance(requirement, bt.AnyRequirement):
+            if requirement is bt.any_requirement:
                 # The lattice type of the constant might be Zero or One; in that case,
                 # generate a bool constant node.
                 required_type = bt.lattice_to_bmg(it)
@@ -103,7 +133,16 @@ class RequirementsFixer:
 
         # We cannot convert this node to any type that meets the requirement.
         # Add an error.
-        self.errors.add_error(Violation(node, it, requirement, consumer, edge))
+        self.errors.add_error(
+            Violation(
+                node,
+                it,
+                requirement,
+                consumer,
+                edge,
+                self.bmg.execution_context.node_locations(consumer),
+            )
+        )
         return node
 
     def _convert_operator_to_atomic_type(
@@ -260,7 +299,9 @@ class RequirementsFixer:
         # meets an upper bound requirement, then the conversion we want exists.
 
         node_type = self._typer[node]
-        if self._type_meets_requirement(node_type, bt.upper_bound(requirement)):
+        if requirement is bt.any_real_matrix:
+            result = self.bmg.add_to_real_matrix(node)
+        elif self._type_meets_requirement(node_type, bt.upper_bound(requirement)):
             # If we got here then the node did NOT meet the requirement,
             # but its type DID meet an upper bound requirement, which
             # implies that the requirement was not an upper bound requirement.
@@ -291,7 +332,14 @@ class RequirementsFixer:
         else:
             # We have no way to make the conversion we need, so add an error.
             self.errors.add_error(
-                Violation(node, node_type, requirement, consumer, edge)
+                Violation(
+                    node,
+                    node_type,
+                    requirement,
+                    consumer,
+                    edge,
+                    self.bmg.execution_context.node_locations(consumer),
+                )
             )
             return node
 
@@ -397,7 +445,8 @@ class RequirementsFixer:
         assert isinstance(node, bn.OperatorNode)
         return self._meet_operator_requirement(node, requirement, consumer, edge)
 
-    def fix_problems(self) -> None:
+    def fix_problems(self) -> bool:
+        made_progress = False
         nodes = self.bmg.all_ancestor_nodes()
         for node in nodes:
             requirements = self._reqs.requirements(node)
@@ -415,3 +464,14 @@ class RequirementsFixer:
                     node_was_updated = True
             if node_was_updated:
                 self._typer.update_type(node)
+                made_progress = True
+        return made_progress
+
+
+def requirements_fixer(bmg: BMGraphBuilder) -> GraphFixer:
+    def graph_fixer():
+        rf = RequirementsFixer(bmg, LatticeTyper())
+        made_progress = rf.fix_problems()
+        return made_progress, rf.errors
+
+    return graph_fixer
