@@ -3,6 +3,7 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+import math
 from typing import Optional
 
 import beanmachine.ppl.compiler.bmg_nodes as bn
@@ -10,19 +11,35 @@ import beanmachine.ppl.compiler.bmg_types as bt
 from beanmachine.ppl.compiler.bm_graph_builder import BMGraphBuilder
 from beanmachine.ppl.compiler.bmg_node_types import is_supported_by_bmg
 from beanmachine.ppl.compiler.bmg_types import PositiveReal
-from beanmachine.ppl.compiler.error_report import BMGError, UnsupportedNode
-from beanmachine.ppl.compiler.fix_problem import ProblemFixerBase
+from beanmachine.ppl.compiler.error_report import (
+    BadMatrixMultiplication,
+    BMGError,
+    UnsupportedNode,
+    UntypableNode,
+)
+from beanmachine.ppl.compiler.fix_problem import (
+    edge_error_pass,
+    GraphFixer,
+    node_error_pass,
+    node_fixer_first_match,
+    NodeFixer,
+    type_guard,
+)
 from beanmachine.ppl.compiler.graph_labels import get_edge_label
 from beanmachine.ppl.compiler.lattice_typer import LatticeTyper
 
 
-class UnsupportedNodeFixer(ProblemFixerBase):
+class UnsupportedNodeFixer:
     """This class takes a Bean Machine Graph builder and attempts to
     fix all uses of unsupported operators by replacing them with semantically
     equivalent nodes that are supported by BMG."""
 
+    _bmg: BMGraphBuilder
+    _typer: LatticeTyper
+
     def __init__(self, bmg: BMGraphBuilder, typer: LatticeTyper) -> None:
-        ProblemFixerBase.__init__(self, bmg, typer)
+        self._bmg = bmg
+        self._typer = typer
 
     def _replace_division(self, node: bn.DivisionNode) -> Optional[bn.BMGNode]:
         # BMG has no division node. We replace division by a constant with
@@ -43,6 +60,34 @@ class UnsupportedNodeFixer(ProblemFixerBase):
         powr = self._bmg.add_power(r, neg1)
         return self._bmg.add_multiplication(node.left, powr)
 
+    def _replace_exp2(self, node: bn.Exp2Node) -> bn.BMGNode:
+        two = self._bmg.add_constant(2.0)
+        return self._bmg.add_power(two, node.operand)
+
+    def _replace_log10(self, node: bn.Log10Node) -> bn.BMGNode:
+        # log10(x) = log(x) * (1/log(10))
+        c = self._bmg.add_constant(1.0 / math.log(10))
+        ln = self._bmg.add_log(node.operand)
+        return self._bmg.add_multiplication(ln, c)
+
+    def _replace_log1p(self, node: bn.Log1pNode) -> bn.BMGNode:
+        # log1p(x) = log(1+x)
+        # TODO: If we added a log1p node to BMG then it could get
+        # more accurate results.
+        one = self._bmg.add_constant(1.0)
+        add = self._bmg.add_addition(one, node.operand)
+        return self._bmg.add_log(add)
+
+    def _replace_log2(self, node: bn.Log10Node) -> bn.BMGNode:
+        # log2(x) = log(x) * (1/log(2))
+        c = self._bmg.add_constant(1.0 / math.log(2))
+        ln = self._bmg.add_log(node.operand)
+        return self._bmg.add_multiplication(ln, c)
+
+    def _replace_squareroot(self, node: bn.SquareRootNode) -> bn.BMGNode:
+        half = self._bmg.add_constant(0.5)
+        return self._bmg.add_power(node.operand, half)
+
     def _replace_uniform(self, node: bn.UniformNode) -> Optional[bn.BMGNode]:
         # TODO: Suppose we have something like Uniform(1.0, 2.0).  Can we replace that
         # with (Flat() + 1.0) ? The problem is that if there is an observation on
@@ -166,6 +211,9 @@ class UnsupportedNodeFixer(ProblemFixerBase):
         # Otherwise, just generate the vector LSE BMG node.
         return self._bmg.add_logsumexp_vector(operand)
 
+    def _replace_lae(self, node: bn.LogAddExpNode) -> Optional[bn.BMGNode]:
+        return self._bmg.add_logsumexp(*node.inputs)
+
     def _replace_tensor(self, node: bn.TensorNode) -> Optional[bn.BMGNode]:
         # Replace a 1-d or 2-d tensor with a TO_MATRIX node.
         size = node._size
@@ -277,121 +325,114 @@ class UnsupportedNodeFixer(ProblemFixerBase):
         # turn into BMG nodes.
         return None
 
-    def _fix_matrix_scale(self, n: bn.BMGNode) -> bn.BMGNode:
-        # Double check we can proceed, and that exactly one is scalar
-        assert self._fixable_matrix_scale(n)
-        # We'll need to name the inputs and their types
-        left, right = n.inputs
-        left_type, right_type = [self._typer[i] for i in n.inputs]
-        # Let's assume the first is the scalr one
-        scalar, matrix = left, right
-        # Fix it if necessary
-        if right_type.is_singleton():
-            scalar = right
-            matrix = left
-        return self._bmg.add_matrix_scale(scalar, matrix)
 
-    def _get_replacement(self, n: bn.BMGNode) -> Optional[bn.BMGNode]:
-        # TODO:
-        # Not -> Complement
-        if isinstance(n, bn.Chi2Node):
-            return self._replace_chi2(n)
-        if isinstance(n, bn.DivisionNode):
-            return self._replace_division(n)
-        if isinstance(n, bn.IndexNode):
-            return self._replace_index(n)
-        if isinstance(n, bn.ItemNode):
-            return self._replace_item(n)
-        if isinstance(n, bn.LogSumExpTorchNode):
-            return self._replace_lse(n)
-        if isinstance(n, bn.SwitchNode):
-            return self._replace_switch(n)
-        if isinstance(n, bn.TensorNode):
-            return self._replace_tensor(n)
-        if isinstance(n, bn.UniformNode):
-            return self._replace_uniform(n)
+def unsupported_node_fixer(bmg: BMGraphBuilder, typer: LatticeTyper) -> NodeFixer:
+    usnf = UnsupportedNodeFixer(bmg, typer)
+    return node_fixer_first_match(
+        [
+            type_guard(bn.Chi2Node, usnf._replace_chi2),
+            type_guard(bn.DivisionNode, usnf._replace_division),
+            type_guard(bn.Exp2Node, usnf._replace_exp2),
+            type_guard(bn.IndexNode, usnf._replace_index),
+            type_guard(bn.ItemNode, usnf._replace_item),
+            type_guard(bn.Log10Node, usnf._replace_log10),
+            type_guard(bn.Log1pNode, usnf._replace_log1p),
+            type_guard(bn.Log2Node, usnf._replace_log2),
+            type_guard(bn.LogSumExpTorchNode, usnf._replace_lse),
+            type_guard(bn.LogAddExpNode, usnf._replace_lae),
+            type_guard(bn.SquareRootNode, usnf._replace_squareroot),
+            type_guard(bn.SwitchNode, usnf._replace_switch),
+            type_guard(bn.TensorNode, usnf._replace_tensor),
+            type_guard(bn.UniformNode, usnf._replace_uniform),
+        ]
+    )
 
-        # See note in _needs_fixing below.
-        if isinstance(n, bn.MultiplicationNode):
-            return self._fix_matrix_scale(n)
 
-        return None
+# TODO: We should make a rewriter that detects stochastic index
+# into list.  We will need to detect if the list is (1) all
+# numbers, in which case we can make a constant matrix out of it,
+# (2) mix of numbers and stochastic elements, in which case we can
+# make it into a TO_MATRIX node, or (3) wrong shape or contents,
+# in which case we must give an error.  We will likely want to
+# move this check for unsupported constant value to AFTER that rewrite.
 
-    def _fixable_matrix_scale(self, n: bn.BMGNode) -> bool:
-        # A matrix multiplication is fixable (to matrix_scale) if it is
-        # a binary multiplication with non-singlton result type
-        # and the type of one argument is matrix and the other is scalar
-        if not isinstance(n, bn.MultiplicationNode) or not (len(n.inputs) == 2):
-            return False
-        # The return type of the node should be matrix
-        if not (self._typer[n]).is_singleton():
-            return False
-        # Now let's check the types of the inputs
-        input_types = [self._typer[i] for i in n.inputs]
-        # If both are scalar, then there is nothing to do
-        if all(t.is_singleton() for t in input_types):
-            return False  # Both are scalar
-        # If both are matrices, then there is nothing to do
-        if all(not (t.is_singleton()) for t in input_types):
-            return False  # Both are matrices
-        return True
 
-    def _needs_fixing(self, n: bn.BMGNode) -> bool:
-        # Constants that can be converted to constant nodes of the appropriate type
-        # will be converted in the requirements checking pass. For now, just detect
-        # constants that cannot possibly be supported because they are the wrong
-        # dimensionality. We will fail to fix it in _get_replacement and report an error.
-
-        # TODO: We should make a rewriter that detects stochastic index
-        # into list.  We will need to detect if the list is (1) all
-        # numbers, in which case we can make a constant matrix out of it,
-        # (2) mix of numbers and stochastic elements, in which case we can
-        # make it into a TO_MATRIX node, or (3) wrong shape or contents,
-        # in which case we must give an error.  We will likely want to
-        # move this check for unsupported constant value to AFTER that rewrite.
-
-        if isinstance(n, bn.ConstantNode):
-            t = bt.type_of_value(n.value)
-            return t == bt.Tensor or t == bt.Untypable
-
-        # TODO: We have an ordering problem in the fixers:
-        # Consider a model with (tensor([[c(), c()],[c(), c()]]) * 10.0).exp() in it,
-        # where c() is a sample from Chi2(1.0).
-        #
-        # The devectorizer skips rewriting the tensor multiplied by scalar operation
-        # because matrix scale rewriter will do so. However it does devectorize the
-        # exp(); it adds index ops to extract the four values from the multiplication.
-        #
-        # Now, which runs first, the unsupported node fixer or the matrix scale fixer?
-        #
-        # * Unsupported node fixer must run first so that the Chi2(1.0) is rewritten into
-        #   supported Gamma(0.5, 0.5) before the matrix scale fixer needs to know the type
-        #   of the multiplication. But...
-        # * Matrix scale fixer must run first so that unsupported node fixer can correctly
-        #   rewrite and optimize the index operations.
-        #
-        # We have a chicken-and-egg problem here.
-        #
-        # The long-term solution is:
-        #
-        # * Extract the error reporting functionality from the unsupported node rewriter
-        #   into its own pass.  Unsupported node rewriter just makes a best effort to rewrite.
-        # * All rewriting passes run in turn making best efforts to rewrite until a fixpoint is
-        #   reached; THEN we check for remaining errors.
-        #
-        # However, that principled solution will require some minor rearchitecting of the
-        # graph rewriters. For now, what we'll do is move the matrix scale fixer INTO the
-        # unsupported node fixer. Since we rewrite from ancestors to descendants, we'll
-        # rewrite the chi2, and then the multiplication, and then the index nodes, which is
-        # the order we need to do them in.
-        if self._fixable_matrix_scale(n):
-            return True
-
-        # It's not a constant. If the node is not supported then try to fix it.
-        return not is_supported_by_bmg(n)
-
-    def _get_error(self, n: bn.BMGNode, index: int) -> Optional[BMGError]:
+def unsupported_node_reporter(bmg: BMGraphBuilder) -> GraphFixer:
+    def _error_for_unsupported_node(n: bn.BMGNode, index: int) -> Optional[BMGError]:
         # TODO: The edge labels used to visualize the graph in DOT
         # are not necessarily the best ones for displaying errors.
         # Consider fixing this.
-        return UnsupportedNode(n.inputs[index], n, get_edge_label(n, index))
+
+        # Constants that can be converted to constant nodes of the appropriate type
+        # will be converted in the requirements checking pass. Here we just detect
+        # constants that cannot possibly be supported because they are the wrong
+        # dimensionality.
+
+        parent = n.inputs[index]
+
+        if isinstance(parent, bn.ConstantNode):
+            t = bt.type_of_value(parent.value)
+            if t != bt.Tensor and t != bt.Untypable:
+                return None
+
+        if is_supported_by_bmg(parent):
+            return None
+
+        return UnsupportedNode(
+            parent,
+            n,
+            get_edge_label(n, index),
+            bmg.execution_context.node_locations(parent),
+        )
+
+    return edge_error_pass(bmg, _error_for_unsupported_node)
+
+
+def bad_matmul_reporter(bmg: BMGraphBuilder) -> GraphFixer:
+    typer = LatticeTyper()
+
+    def get_error(node: bn.BMGNode) -> Optional[BMGError]:
+        if not isinstance(node, bn.MatrixMultiplicationNode):
+            return None
+
+        # If an operand is bad then we'll produce an error later. This
+        # pass just looks for row/column mismatches.
+        lt = typer[node.left]
+        if not isinstance(lt, bt.BMGMatrixType):
+            return None
+        rt = typer[node.right]
+        if not isinstance(rt, bt.BMGMatrixType):
+            return None
+        if lt.columns == rt.rows:
+            return None
+        return BadMatrixMultiplication(
+            node, lt, rt, bmg.execution_context.node_locations(node)
+        )
+
+    return node_error_pass(bmg, get_error)
+
+
+def untypable_node_reporter(bmg: BMGraphBuilder) -> GraphFixer:
+    # By the time this pass runs every node in the graph should
+    # be a valid BMG node (except constants, which are rewritten later)
+    # and therefore should have a valid type. Later passes rely on this
+    # invariant, so if it is violated, let's find out now and stop
+    # the compiler rather than producing some strange error later on.
+    typer = LatticeTyper()
+
+    def get_error(node: bn.BMGNode) -> Optional[BMGError]:
+        # If a node is untypable then all its descendants will be too.
+        # We do not want to produce a cascading error here so let's just
+        # report the nodes that are untypable who have no untypable parents.
+        if isinstance(node, bn.ConstantNode):
+            return None
+        t = typer[node]
+        if t != bt.Untypable and t != bt.Tensor:
+            return None
+        for i in node.inputs:
+            t = typer[i]
+            if t == bt.Untypable or t == bt.Tensor:
+                return None
+        return UntypableNode(node, bmg.execution_context.node_locations(node))
+
+    return node_error_pass(bmg, get_error)
