@@ -129,6 +129,7 @@ _constant_matrix_graph_types: Dict[type, bt.BMGMatrixType] = {
 # These are the node types which always represent a matrix in BMG.
 # Even if the node is a 1x1 matrix, it is a matrix and not an atomic value.
 _always_matrix_types: Set[type] = {
+    bn.CholeskyNode,
     bn.ColumnIndexNode,
     bn.ConstantBooleanMatrixNode,
     bn.ConstantNaturalMatrixNode,
@@ -139,6 +140,7 @@ _always_matrix_types: Set[type] = {
     bn.ConstantSimplexMatrixNode,
     bn.ToMatrixNode,
     bn.MatrixScaleNode,
+    bn.TransposeNode,
 }
 
 
@@ -155,14 +157,16 @@ class LatticeTyper(TyperBase[bt.BMGLatticeType]):
             # Operators
             bn.AdditionNode: self._type_addition,
             bn.ChoiceNode: self._type_choice,
+            bn.CholeskyNode: self._type_cholesky,
             bn.ColumnIndexNode: self._type_column_index,
             bn.ComplementNode: self._type_complement,
             bn.ExpM1Node: self._type_expm1,
             bn.ExpNode: self._type_exp,
             bn.IfThenElseNode: self._type_if,
             bn.LogNode: self._type_log,
-            bn.MultiplicationNode: self._type_multiplication,
+            bn.MatrixMultiplicationNode: self._type_matrix_multiplication,
             bn.MatrixScaleNode: self._type_matrix_scale,
+            bn.MultiplicationNode: self._type_multiplication,
             bn.NegateNode: self._type_negate,
             bn.PowerNode: self._type_power,
             bn.SampleNode: self._type_sample,
@@ -170,6 +174,7 @@ class LatticeTyper(TyperBase[bt.BMGLatticeType]):
             bn.ToPositiveRealMatrixNode: self._type_to_pos_real_matrix,
             bn.ToRealMatrixNode: self._type_to_real_matrix,
             bn.VectorIndexNode: self._type_index,
+            bn.TransposeNode: self._type_transpose,
         }
 
     def _type_observation(self, node: bn.Observation) -> bt.BMGLatticeType:
@@ -259,6 +264,10 @@ class LatticeTyper(TyperBase[bt.BMGLatticeType]):
             result = bt.Boolean
         return result
 
+    def _type_cholesky(self, node: bn.CholeskyNode) -> bt.BMGLatticeType:
+        # TODO: Check to see if the input is a square matrix.
+        return self[node.operand]
+
     def _type_index(self, node: bn.VectorIndexNode) -> bt.BMGLatticeType:
         # The lattice type of an index is derived from the lattice type of
         # the vector, but it's not as straightforward as just
@@ -291,6 +300,21 @@ class LatticeTyper(TyperBase[bt.BMGLatticeType]):
             return it
         return bt.Real
 
+    def _type_matrix_multiplication(
+        self, node: bn.MatrixMultiplicationNode
+    ) -> bt.BMGLatticeType:
+        assert len(node.inputs) == 2
+        lt = self[node.left]
+        assert lt is not bt.Untypable
+        assert isinstance(lt, bt.BMGMatrixType)
+        rt = self[node.right]
+        assert rt is not bt.Untypable
+        assert isinstance(rt, bt.BMGMatrixType)
+        # Note that we do not detect here if lt.columns != rt.rows, which would
+        # be illegal. We assume the type of the output is a real matrix with
+        # lt.rows and rt.columns. That error condition will be checked elsewhere.
+        return bt.RealMatrix(lt.rows, rt.columns)
+
     def _type_matrix_scale(self, node: bn.MatrixScaleNode) -> bt.BMGLatticeType:
         assert len(node.inputs) == 2
         lt = self[node.left]
@@ -369,6 +393,14 @@ class LatticeTyper(TyperBase[bt.BMGLatticeType]):
         assert self.is_matrix(op)
         return bt.PositiveRealMatrix(t.rows, t.columns)
 
+    def _type_transpose(self, node: bn.TransposeNode) -> bt.BMGLatticeType:
+        op = node.operand
+        t = self[op]
+        assert t is not bt.Untypable
+        assert isinstance(t, bt.BMGMatrixType)
+        assert self.is_matrix(op)
+        return bt.RealMatrix(t.columns, t.rows)
+
     def _compute_type_inputs_known(self, node: bn.BMGNode) -> bt.BMGLatticeType:
         # If there is any input node whose type cannot be determined, then *none*
         # of its descendents can be determined, even if a descendent node always
@@ -408,6 +440,18 @@ class LatticeTyper(TyperBase[bt.BMGLatticeType]):
         t = self[node]
         return t != bt.Untypable and bt.supremum(t, bt.Probability) == bt.Probability
 
+    def is_neg_real(self, node: bn.BMGNode) -> bool:
+        t = self[node]
+        return t != bt.Untypable and bt.supremum(t, bt.NegativeReal) == bt.NegativeReal
+
+    def is_pos_real(self, node: bn.BMGNode) -> bool:
+        t = self[node]
+        return t != bt.Untypable and bt.supremum(t, bt.PositiveReal) == bt.PositiveReal
+
+    def is_real(self, node: bn.BMGNode) -> bool:
+        t = self[node]
+        return t != bt.Untypable and bt.supremum(t, bt.Real) == bt.Real
+
     def is_matrix(self, node: bn.BMGNode) -> bool:
         t = type(node)
         if t in _always_matrix_types:
