@@ -3,8 +3,9 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+from beanmachine.ppl.distributions.delta import Delta
 from beanmachine.ppl.distributions.flat import Flat
 from beanmachine.ppl.distributions.unit import Unit
 
 
-__all__ = ["Flat", "Unit"]
+__all__ = ["Delta", "Flat", "Unit"]
