@@ -8,16 +8,16 @@ import torch.distributions as dist
 from torch import Tensor
 
 
-class GammaGammaModel(object):
-    def __init__(self, shape: Tensor, rate: Tensor, alpha: Tensor):
+class GammaGammaModel:
+    def __init__(self, shape: Tensor, rate: Tensor, alpha: Tensor) -> None:
         self.shape_ = shape
         self.rate_ = rate
         self.alpha_ = alpha
 
     @bm.random_variable
-    def gamma_p(self):
+    def gamma_p(self) -> dist.Distribution:
         return dist.Gamma(self.shape_, self.rate_)
 
     @bm.random_variable
-    def gamma(self):
+    def gamma(self) -> dist.Distribution:
         return dist.Gamma(self.alpha_, self.gamma_p())
