@@ -9,16 +9,17 @@ import torch.distributions as dist
 from torch import Tensor
 
 
-class GammaNormalModel(object):
-    def __init__(self, shape: Tensor, rate: Tensor, mu: Tensor):
+class GammaNormalModel:
+    def __init__(self, shape: Tensor, rate: Tensor, mu: Tensor) -> None:
         self.shape_ = shape
         self.rate_ = rate
         self.mu_ = mu
 
     @bm.random_variable
-    def gamma(self):
+    def gamma(self) -> dist.Distribution:
         return dist.Gamma(self.shape_, self.rate_)
 
     @bm.random_variable
-    def normal(self):
+    def normal(self) -> dist.Distribution:
+        # pyre-fixme[58]: `/` is not supported for operand types `int` and `Tensor`.
         return dist.Normal(self.mu_, 1 / torch.sqrt(self.gamma()))
