@@ -6,34 +6,26 @@
 import inspect
 from collections import defaultdict
 from typing import (
-    Dict,
-    Tuple,
     Callable,
-    Union,
-    List,
-    Set,
-    Optional,
-    TYPE_CHECKING,
     cast,
+    Dict,
+    List,
+    Optional,
+    Set,
+    Tuple,
+    TYPE_CHECKING,
+    Union,
 )
 
-import torch.distributions as dist
 from beanmachine.ppl.inference.base_inference import BaseInference
-from beanmachine.ppl.inference.proposer.base_proposer import (
-    BaseProposer,
-)
-from beanmachine.ppl.inference.proposer.sequential_proposer import (
-    SequentialProposer,
-)
+from beanmachine.ppl.inference.proposer.base_proposer import BaseProposer
+from beanmachine.ppl.inference.proposer.nuts_proposer import NUTSProposer
+from beanmachine.ppl.inference.proposer.sequential_proposer import SequentialProposer
 from beanmachine.ppl.inference.proposer.single_site_uniform_proposer import (
     SingleSiteUniformProposer,
 )
-from beanmachine.ppl.inference.single_site_nmc import (
-    SingleSiteNewtonianMonteCarlo,
-)
 from beanmachine.ppl.model.rv_identifier import RVIdentifier
 from beanmachine.ppl.world import World
-from beanmachine.ppl.world.utils import is_constraint_eq
 
 if TYPE_CHECKING:
     from enum import Enum
@@ -44,12 +36,21 @@ if TYPE_CHECKING:
         def __iter__(self):
             pass
 
-    Ellipsis = EllipsisClass.Ellipsis
+    Ellipsis: EllipsisClass = EllipsisClass.Ellipsis
 else:
     EllipsisClass = type(Ellipsis)
 
 
-class _DefaultInference(SingleSiteNewtonianMonteCarlo):
+class _DefaultInference(BaseInference):
+    """
+    Mixed inference class that handles both discrete and continuous RVs
+    """
+
+    def __init__(self):
+        self._disc_proposers = {}
+        self._cont_proposer = None
+        self._continuous_rvs = set()
+
     def get_proposers(
         self,
         world: World,
@@ -58,22 +59,28 @@ class _DefaultInference(SingleSiteNewtonianMonteCarlo):
     ) -> List[BaseProposer]:
         proposers = []
         for node in target_rvs:
-            if node not in self._proposers:
+            if node not in self._disc_proposers:
                 support = world.get_variable(node).distribution.support
-                if any(
-                    is_constraint_eq(
-                        support,
-                        (
-                            dist.constraints.real,
-                            dist.constraints.simplex,
-                            dist.constraints.greater_than,
-                        ),
-                    )
-                ):
-                    self._proposers[node] = self._init_nmc_proposer(node, world)
+                if not support.is_discrete:
+                    self._continuous_rvs.add(node)
+                    continue
                 else:
-                    self._proposers[node] = SingleSiteUniformProposer(node)
-            proposers.append(self._proposers[node])
+                    self._disc_proposers[node] = SingleSiteUniformProposer(node)
+            proposers.append(self._disc_proposers[node])
+        if self._cont_proposer is not None:
+            if len(self._cont_proposer._target_rvs) != len(self._continuous_rvs):
+                raise ValueError(
+                    "Graph has changed between iterations. NUTS requires a"
+                    " static model."
+                )
+            proposers.append(self._cont_proposer)
+        else:
+            if len(self._continuous_rvs):
+                continuous_proposer = NUTSProposer(
+                    world, self._continuous_rvs, num_adaptive_sample
+                )
+                self._cont_proposer = continuous_proposer
+                proposers.append(self._cont_proposer)
         return proposers
 
 
@@ -103,8 +110,9 @@ def _get_nodes_for_rv_family(
 class CompositionalInference(BaseInference):
     """
     The ``CompositionalInference`` class enables combining multiple inference algorithms
-    and blocking random variables together. By default, it uses different proposer to
-    update each site by (by look at the support of the distribution that that site).
+    and blocking random variables together. By default, continuous variables will be
+    blocked together and use the ``GlobalNoUTurnProposer``. Discrete variables will
+    be proposed independently with ``SingleSiteUniformProposer``.
     To override the default behavior, you can pass an ``inference_dict``. To learn more
     about Compositional Inference, please see the `Compositional Inference
     <https://beanmachine.org/docs/compositional_inference/>`_ page on our website.
@@ -124,6 +132,10 @@ class CompositionalInference(BaseInference):
 
         CompositionalInference({(model.foo, model.bar): bm.GlobalNoUTurnSampler()})
 
+    .. warning::
+        When using the default inference behavior, graphs (i.e. the number of latent variables)
+        must be static and cannot change between iterations.
+
     Args:
         inference_dict: an optional inference configuration as shown above.
     """
@@ -182,6 +194,19 @@ class CompositionalInference(BaseInference):
 
         self._default_inference = default_inference
 
+    def _get_default_num_adaptive_samples(self, num_samples: int) -> int:
+        """Returns the default number of adaptive samples for CompositionalInference,
+        which equals to the maximum number of adaptive samples recommended by each
+        algorithm in the inference config."""
+        num_adaptive_per_algorithm = [
+            self._default_inference._get_default_num_adaptive_samples(num_samples)
+        ]
+        for inference in self.config.values():
+            num_adaptive_per_algorithm.append(
+                inference._get_default_num_adaptive_samples(num_samples)
+            )
+        return max(num_adaptive_per_algorithm)
+
     def get_proposers(
         self,
         world: World,
@@ -194,10 +219,10 @@ class CompositionalInference(BaseInference):
             rv_family_to_node[node.wrapper].add(node)
 
         all_proposers = []
-        for target_families, inferences in self.config.items():
+        for target_families, inference in self.config.items():
             nodes = _get_nodes_for_rv_family(target_families, rv_family_to_node)
             if len(nodes) > 0:
-                proposers = inferences.get_proposers(world, nodes, num_adaptive_sample)
+                proposers = inference.get_proposers(world, nodes, num_adaptive_sample)
                 if isinstance(target_families, tuple):
                     # tuple of RVs == block into a single accept/reject step
                     proposers = [SequentialProposer(proposers)]
