@@ -4,7 +4,7 @@
 # LICENSE file in the root directory of this source tree.
 
 import torch.distributions as dist
-from beanmachine.ppl.inference.proposer.base_single_site_proposer import (
+from beanmachine.ppl.inference.proposer.base_single_site_mh_proposer import (
     BaseSingleSiteMHProposer,
 )
 from beanmachine.ppl.world import World
