@@ -27,11 +27,13 @@ class SingleSiteRandomWalkProposer(SingleSiteAncestralProposer):
         if torch.isnan(accept_log_prob):
             return
         accept_prob = accept_log_prob.exp()
-        if world[self.node].shape[0] == 1:
+        val_shape = world[self.node].shape
+        if len(val_shape) == 0 or val_shape[0] == 1:
             target_acc_rate = self.target_acc_rate[False]
             c = torch.reciprocal(target_acc_rate)
         else:
             target_acc_rate = self.target_acc_rate[True]
+            # pyre-fixme[6]: For 1st param expected `Tensor` but got `float`.
             c = torch.reciprocal(1.0 - target_acc_rate)
 
         new_step_size = self.step_size * torch.exp(
@@ -48,10 +50,15 @@ class SingleSiteRandomWalkProposer(SingleSiteAncestralProposer):
 
         if is_constraint_eq(node_support, dist.constraints.real):
             return dist.Normal(node.value, self.step_size)
-        elif is_constraint_eq(node_support, dist.constraints.greater_than):
+        elif any(
+            is_constraint_eq(
+                node_support,
+                (dist.constraints.greater_than, dist.constraints.greater_than_eq),
+            )
+        ):
             lower_bound = node_support.lower_bound
             proposal_distribution = self.gamma_dist_from_moments(
-                node.value - lower_bound, self.step_size ** 2
+                node.value - lower_bound, self.step_size**2
             )
             transform = dist.AffineTransform(loc=lower_bound, scale=1.0)
             transformed_proposal = dist.TransformedDistribution(
@@ -90,7 +97,7 @@ class SingleSiteRandomWalkProposer(SingleSiteAncestralProposer):
         :param sigma: sigma value
         :returns: returns the Beta distribution given mu and sigma.
         """
-        beta = expectation / (sigma ** 2)
+        beta = expectation / (sigma**2)
         beta = torch.clamp(beta, min=1e-3)
         alpha = expectation * beta
         alpha = torch.clamp(alpha, min=1e-3)
@@ -111,7 +118,7 @@ class SingleSiteRandomWalkProposer(SingleSiteAncestralProposer):
         https://stats.stackexchange.com/questions/12232/calculating-the-
         parameters-of-a-beta-distribution-using-the-mean-and-variance
         """
-        alpha = ((1.0 - mu) / (sigma ** 2) - (1.0 / mu)) * (mu ** 2)
+        alpha = ((1.0 - mu) / (sigma**2) - (1.0 / mu)) * (mu**2)
         beta = alpha * (1.0 / mu - 1.0)
         distribution = dist.Beta(concentration1=alpha, concentration0=beta)
         return distribution
@@ -128,5 +135,5 @@ class SingleSiteRandomWalkProposer(SingleSiteAncestralProposer):
         :param sigma: sigma value
         :returns: returns the Dirichlet distribution given mu and sigma.
         """
-        alpha = mu / (torch.norm(mu) * sigma ** 2)
+        alpha = mu / (torch.norm(mu) * sigma**2)
         return dist.Dirichlet(concentration=alpha)
