@@ -6,9 +6,7 @@
 from beanmachine.ppl.inference.proposer.single_site_uniform_proposer import (
     SingleSiteUniformProposer,
 )
-from beanmachine.ppl.inference.single_site_inference import (
-    SingleSiteInference,
-)
+from beanmachine.ppl.inference.single_site_inference import SingleSiteInference
 
 
 class SingleSiteUniformMetropolisHastings(SingleSiteInference):
