@@ -8,33 +8,18 @@ from beanmachine.ppl.legacy.inference.rejection_sampling_infer import RejectionS
 from beanmachine.ppl.legacy.inference.single_site_ancestral_mh import (
     SingleSiteAncestralMetropolisHastings,
 )
-from beanmachine.ppl.legacy.inference.single_site_hamiltonian_monte_carlo import (
-    SingleSiteHamiltonianMonteCarlo,
-)
 from beanmachine.ppl.legacy.inference.single_site_newtonian_monte_carlo import (
     SingleSiteNewtonianMonteCarlo,
 )
-from beanmachine.ppl.legacy.inference.single_site_no_u_turn_sampler import (
-    SingleSiteNoUTurnSampler,
-)
 from beanmachine.ppl.legacy.inference.single_site_random_walk import (
     SingleSiteRandomWalk,
 )
-from beanmachine.ppl.legacy.inference.single_site_uniform_mh import (
-    SingleSiteUniformMetropolisHastings,
-)
 
 
 __all__ = [
     "CompositionalInference",
     "RejectionSampling",
     "SingleSiteAncestralMetropolisHastings",
-    "SingleSiteHamiltonianMonteCarlo",
     "SingleSiteNewtonianMonteCarlo",
-    "SingleSiteNoUTurnSampler",
     "SingleSiteRandomWalk",
-    "SingleSiteUniformMetropolisHastings",
-    "Predictive",
-    "empirical",
-    "simulate",
 ]
