@@ -5,14 +5,10 @@
 
 import copy
 from collections import defaultdict
-from typing import Callable, Dict, List, Optional, Set, Tuple
+from typing import Dict, List, Optional, Set, Tuple
 
 import torch
 import torch.distributions as dist
-import torch.nn as nn
-from beanmachine.ppl.experimental.vi.mean_field_variational_approximation import (
-    MeanFieldVariationalApproximation,
-)
 from beanmachine.ppl.legacy.world.diff import Diff
 from beanmachine.ppl.legacy.world.diff_stack import DiffStack
 from beanmachine.ppl.legacy.world.variable import TransformData, TransformType, Variable
@@ -88,10 +84,6 @@ class World(BaseWorld):
     )
     """
 
-    vi_dicts: Optional[Callable[[RVIdentifier], MeanFieldVariationalApproximation]]
-    params_: Dict[RVIdentifier, nn.Parameter]
-    model_to_guide_ids_: Optional[Dict[RVIdentifier, RVIdentifier]]
-
     def __init__(self):
         self.variables_ = WorldVars()
         self.stack_ = []
@@ -103,9 +95,6 @@ class World(BaseWorld):
         self.maintain_graph_ = True
         self.cache_functionals_ = False
         self.cached_functionals_ = defaultdict()
-        self.vi_dicts = None
-        self.params_ = {}
-        self.model_to_guide_ids_ = None
 
     def set_initialize_from_prior(self, initialize_from_prior: bool = True):
         """
@@ -697,16 +686,6 @@ class World(BaseWorld):
             proposed_score,
         )
 
-    def get_param(self, param: RVIdentifier) -> nn.Parameter:
-        "Gets a parameter or initializes it if not found."
-        if param not in self.params_:
-            self.params_[param] = nn.Parameter(param.function(*param.arguments))
-        return self.params_[param]
-
-    def set_params(self, params: Dict[RVIdentifier, nn.Parameter]):
-        "Sets the parameters in this World to specified values."
-        self.params_ = params
-
     def update_graph(self, node: RVIdentifier) -> Tensor:
         """
         Updates the parents and children of the node based on the stack
@@ -764,37 +743,8 @@ class World(BaseWorld):
 
         obs_value = self.observations_.get(node)
 
-        # resample latents from q
-        value = None
-        vi_dicts = self.vi_dicts
-        model_to_guide_ids = self.model_to_guide_ids_
-        if obs_value is None:
-            # TODO: messy, consider strategy pattern
-            if vi_dicts is not None:
-                # mean-field VI
-                variational_approx = vi_dicts(node)
-                value = variational_approx.rsample((1,)).squeeze()
-            elif (
-                isinstance(model_to_guide_ids, dict)
-                and node not in model_to_guide_ids.values()  # is not a model RV
-            ):
-                # guide-based VI on non-guide nodes only
-                assert (
-                    node in model_to_guide_ids
-                ), f"Could not find a guide for {node}. VariationalInference requires every latent variable in the model to have a corresponding guide."
-                guide_node = model_to_guide_ids[node]
-                guide_var = self.get_node_in_world(guide_node)
-                if not guide_var:
-                    # initialize guide node if missing
-                    self.call(guide_node)
-                guide_var = self.get_node_in_world_raise_error(guide_node)
-                try:
-                    value = guide_var.distribution.rsample(torch.Size((1,)))
-                except NotImplementedError:
-                    value = guide_var.distribution.sample(torch.Size((1,)))
-
         node_var.update_fields(
-            value,
+            None,
             obs_value,
             self.get_transforms_for_node(node),
             self.get_proposer_for_node(node),
